# Copyright (C) 2006-2009 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from ClusterModel.Ip import Ip
from ClusterModel.Fs import Fs
from ClusterModel.Clusterfs import Clusterfs
from ClusterModel.Netfs import Netfs
from ClusterModel.NFSExport import NFSExport
from ClusterModel.NFSClient import NFSClient
from ClusterModel.Script import Script
from ClusterModel.Samba import Samba
from ClusterModel.Tomcat5 import Tomcat5
from ClusterModel.Postgres8 import Postgres8
from ClusterModel.Apache import Apache
from ClusterModel.OpenLDAP import OpenLDAP
from ClusterModel.LVM import LVM
from ClusterModel.MySQL import MySQL
from ClusterModel.SAPDatabase import SAPDatabase
from ClusterModel.SAPInstance import SAPInstance
from ClusterModel.OracleDB import OracleDB
from ClusterModel.OracleInstance import OracleInstance
from ClusterModel.OracleListener import OracleListener
from ClusterModel.SybaseASE import SybaseASE

from LuciZope import GetFormVars
from LuciSyslog import get_logger
from conga_constants import LUCI_DEBUG_MODE

luci_log = get_logger()

def get_fsid_list(model):
	obj_list = model.searchObjectTree('fs')
	obj_list.extend(model.searchObjectTree('clusterfs'))
	return map(lambda x: x.getAttribute('fsid') and int(x.getAttribute('fsid')) or 0, obj_list)

def fsid_is_unique(model, fsid):
	fsid_list = get_fsid_list(model)
	return fsid not in fsid_list

def generate_fsid(model, name):
	import binascii
	from random import randrange
	fsid_list = get_fsid_list(model)

	fsid = binascii.crc32(name) & 0xffff
	dupe = fsid in fsid_list
	while dupe is True:
		fsid = (fsid + randrange(1, 0xfffe)) & 0xffff
		dupe = fsid in fsid_list
	return fsid

def getResourceForEdit(model, name):
	resPtr = model.getResourcesPtr()
	resources = resPtr.getChildren()

	for res in resources:
		if res.getName() == name:
			resPtr.removeChild(res)
			return res

	if LUCI_DEBUG_MODE is True:
		luci_log.debug_verbose('GRFE0: unable to find resource "%s"' % name)
	raise KeyError, name

def addIp(res, rname, form, model):
	errors = list()

	try:
		addr = form['ip_address'].strip()
		if not addr:
			raise KeyError, 'No IP address was given'
		res.addAttribute('address', addr)
	except KeyError, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addIp4: %s' % err)

	if form.has_key('monitorLink'):
		res.addAttribute('monitor_link', '1')
	else:
		res.addAttribute('monitor_link', '0')

	return errors

def addFs(res, rname, form, model):
	errors = list()

	try:
		mountpoint = form['mountpoint'].strip()
		if not mountpoint:
			raise Exception, 'No mount point was given for "%s"' % rname
		res.addAttribute('mountpoint', mountpoint)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addFs6: %s' % err)

	try:
		device = form['device'].strip()
		if not device:
			raise Exception, 'No device was given for "%s"' % rname
		res.addAttribute('device', device)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addFs7: %s' % err)

	try:
		options = form['options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('options', options)
	except KeyError, e:
		try:
			res.removeAttribute('options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addFs8: %s' % err)

	try:
		fstype = form['fstype'].strip()
		if not fstype:
			raise Exception, 'No filesystem type was given for "%s"' % rname
		res.addAttribute('fstype', fstype)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addFs9: %s' % err)

	try:
		fsid = form['fsid'].strip()
		if not fsid:
			raise Exception, 'no fsid'
		fsid_int = int(fsid)
		if not fsid_is_unique(model, fsid_int):
			raise Exception, 'fsid not uniq'
	except Exception, e:
		fsid = str(generate_fsid(model, rname))
	res.addAttribute('fsid', fsid)

	if form.has_key('forceunmount'):
		res.addAttribute('force_unmount', '1')
	else:
		res.addAttribute('force_unmount', '0')

	if form.has_key('selffence'):
		res.addAttribute('self_fence', '1')
	else:
		res.addAttribute('self_fence', '0')

	if form.has_key('checkfs'):
		res.addAttribute('force_fsck', '1')
	else:
		res.addAttribute('force_fsck', '0')

	return errors

def addClusterfs(res, rname, form, model):
	errors = list()

	try:
		mountpoint = form['mountpoint'].strip()
		if not mountpoint:
			raise Exception, 'No mount point was given for "%s"' % rname
		res.addAttribute('mountpoint', mountpoint)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addGfs5: %s' % err)

	try:
		device = form['device'].strip()
		if not device:
			raise Exception, 'No device was given for "%s"' % rname
		res.addAttribute('device', device)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addGfs6: %s' % err)

	try:
		fstype = form['fstype'].lower().strip()
		if not fstype:
			raise KeyError, 'No filesystem type was given'
		if fstype != 'gfs' and fstype != 'gfs2':
			raise KeyError, 'An invalid file system type of "%s" was given' % fstype
		res.addAttribute('fstype', fstype)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addGfs7a: %s' % err)

	try:
		options = form['options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('options', options)
	except KeyError, e:
		try:
			res.removeAttribute('options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addGfs7: %s' % err)

	try:
		fsid = form['fsid'].strip()
		if not fsid:
			raise Exception, 'no fsid'
		fsid_int = int(fsid)
		if not fsid_is_unique(model, fsid_int):
			raise Exception, 'not uniq'
	except Exception, e:
		fsid = str(generate_fsid(model, rname))
	res.addAttribute('fsid', fsid)

	if form.has_key('forceunmount'):
		res.addAttribute('force_unmount', '1')
	else:
		res.addAttribute('force_unmount', '0')

	if form.has_key('selffence'):
		res.addAttribute('self_fence', '1')
	else:
		res.addAttribute('self_fence', '0')

	return errors

def addNetfs(res, rname, form, model):
	errors = list()

	try:
		mountpoint = form['mountpoint'].strip()
		if not mountpoint:
			raise Exception, 'No mount point was given for "%s"' % rname
		res.addAttribute('mountpoint', mountpoint)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addNfsm5: %s' % err)

	try:
		host = form['host'].strip()
		if not host:
			raise Exception, 'No host server was given for "%s"' % rname
		res.addAttribute('host', host)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addNfsm6 error: %s' % err)

	try:
		options = form['options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('options', options)
	except KeyError, e:
		try:
			res.removeAttribute('options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addNfsm7: %s' % err)

	try:
		exportpath = form['exportpath'].strip()
		if not exportpath:
			raise Exception, 'No export path was given for "%s"' % rname
		res.addAttribute('export', exportpath)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addNfsm8: %s' % err)

	try:
		nfstype = form['nfstype'].strip().lower()
		if nfstype != 'nfs' and nfstype != 'nfs4':
			raise Exception, 'An invalid NFS version "%s" was given for "%s"' \
								% (nfstype, rname)
		res.addAttribute('fstype', nfstype)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addNfsm9: %s' % err)

	if form.has_key('forceunmount'):
		res.addAttribute('force_unmount', '1')
	else:
		res.addAttribute('force_unmount', '0')

	return errors

def addNFSClient(res, rname, form, model):
	errors = list()

	try:
		target = form['target'].strip()
		if not target:
			raise Exception, 'No target was given for "%s"' % rname
		res.addAttribute('target', target)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addNfsc5: %s' % err)

	try:
		options = form['options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('options', options)
	except KeyError, e:
		try:
			res.removeAttribute('options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addNfsc6: %s' % err)

	if form.has_key('allow_recover'):
		res.addAttribute('allow_recover', '1')
	else:
		res.addAttribute('allow_recover', '0')

	return errors

def addNFSExport(res, rname, form, model):
	errors = list()
	return errors

def addScript(res, rname, form, model):
	errors = list()

	try:
		path = form['file'].strip()
		if not path:
			raise Exception, 'No path to a script file was given for this "%s"' % rname
		res.addAttribute('file', path)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addScr5: %s' % err)

	return errors

def addSamba(res, rname, form, model):
	errors = list()

	try:
		workgroup = form['workgroup'].strip()
		if not workgroup:
			raise Exception, 'No workgroup was given for "%s"' % rname
		res.addAttribute('workgroup', workgroup)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addSmb5: %s' % err)

	return errors

def addApache(res, rname, form, model):
	errors = list()

	try:
		server_root = form['server_root'].strip()
		if not server_root:
			raise KeyError, 'No server root was given for "%s"' % rname
		res.addAttribute('server_root', server_root)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addApache5: %s' % err)

	try:
		config_file = form['config_file'].strip()
		if not server_root:
			raise KeyError, 'No path to the Apache configuration file was given for "%s"' % rname
		res.addAttribute('config_file', config_file)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addApache6: %s' % err)

	try:
		options = form['httpd_options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('httpd_options', options)
	except KeyError, e:
		try:
			res.removeAttribute('httpd_options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addApache7: %s' % err)

	try:
		shutdown_wait = int(form['shutdown_wait'].strip())
		res.addAttribute('shutdown_wait', str(shutdown_wait))
	except KeyError, e:
		res.addAttribute('shutdown_wait', '0')
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addApache7: %s' % err)

	return errors

def addMySQL(res, rname, form, model):
	errors = list()

	try:
		config_file = form['config_file'].strip()
		if not config_file:
			raise KeyError, 'No path to the MySQL configuration file was given for "%s"' % rname
		res.addAttribute('config_file', config_file)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addMySQL5: %s' % err)

	try:
		listen_addr = form['listen_address'].strip()
		if not listen_addr:
			raise KeyError, 'No address was given for "%s"' % rname
		res.addAttribute('listen_address', listen_addr)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addMySQL6: %s' % err)

	try:
		options = form['mysql_options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('mysql_options', options)
	except KeyError, e:
		try:
			res.removeAttribute('mysql_options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addMySQL7: %s' % err)

	try:
		shutdown_wait = int(form['shutdown_wait'].strip())
		res.addAttribute('shutdown_wait', str(shutdown_wait))
	except KeyError, e:
		res.addAttribute('shutdown_wait', '0')
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addMySQL7: %s' % err)

	return errors

def addOpenLDAP(res, rname, form, model):
	errors = list()

	try:
		url_list = form['url_list'].strip()
		if not url_list:
			raise KeyError, 'No URL list was given for "%s"' % rname
		res.addAttribute('url_list', url_list)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addOpenLDAP5: %s' % err)

	try:
		config_file = form['config_file'].strip()
		if not config_file:
			raise KeyError, 'No path to the OpenLDAP configuration file was given for "%s"' % rname
		res.addAttribute('config_file', config_file)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addOpenLDAP6: %s' % err)

	try:
		options = form['slapd_options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('slapd_options', options)
	except KeyError, e:
		try:
			res.removeAttribute('slapd_options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addOpenLDAP7: %s' % err)

	try:
		shutdown_wait = int(form['shutdown_wait'].strip())
		res.addAttribute('shutdown_wait', str(shutdown_wait))
	except KeyError, e:
		res.addAttribute('shutdown_wait', '0')
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addOpenLDAP7: %s' % err)

	return errors

def addPostgres8(res, rname, form, model):
	errors = list()
	try:
		user = form['postmaster_user'].strip()
		if not user:
			raise KeyError, 'No postmaster user was given for "%s"' % rname
		res.addAttribute('postmaster_user', user)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addPostgreSQL85: %s' % err)

	try:
		config_file = form['config_file'].strip()
		if not config_file:
			raise KeyError, 'No path to the PostgreSQL 8 configuration file was given for "%s"' % rname
		res.addAttribute('config_file', config_file)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addPostgreSQL86: %s' % err)

	try:
		options = form['postmaster_options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('postmaster_options', options)
	except KeyError, e:
		try:
			res.removeAttribute('postmaster_options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addPostgreSQL87: %s' % err)

	try:
		shutdown_wait = int(form['shutdown_wait'].strip())
		res.addAttribute('shutdown_wait', str(shutdown_wait))
	except KeyError, e:
		res.addAttribute('shutdown_wait', '0')
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addPostgreSQL87: %s' % err)

	return errors

def addTomcat5(res, rname, form, model):
	errors = list()

	try:
		user = form['tomcat_user'].strip()
		if not user:
			raise KeyError, 'No Tomcat user was given for "%s"' % rname
		res.addAttribute('tomcat_user', user)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addTomcat55: %s' % err)

	try:
		config_file = form['config_file'].strip()
		if not config_file:
			raise KeyError, 'No path to the Tomcat 5 configuration file was given for "%s"' % rname
		res.addAttribute('config_file', config_file)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addTomcat56: %s' % err)

	try:
		options = form['catalina_options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('catalina_options', options)
	except KeyError, e:
		try:
			res.removeAttribute('catalina_options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addTomcat57: %s' % err)

	try:
		catalina_base = form['catalina_base'].strip()
		if not catalina_base:
			raise KeyError, 'No cataliny base directory was given for "%s"' % rname
		res.addAttribute('catalina_base', catalina_base)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addTomcat58: %s' % err)

	try:
		shutdown_wait = int(form['shutdown_wait'].strip())
		res.addAttribute('shutdown_wait', str(shutdown_wait))
	except KeyError, e:
		res.addAttribute('shutdown_wait', '0')
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addTomcat59: %s' % err)

	return errors

def addLVM(res, rname, form, model):
	errors = list()

	try:
		vg_name = form['vg_name'].strip()
		if not vg_name:
			raise KeyError, 'No volume group name was given for "%s"' % rname
		res.addAttribute('vg_name', vg_name)
	except Exception, e:
		vg_name = None

	try:
		lv_name = form['lv_name'].strip()
		if not lv_name:
			raise KeyError, 'No logical volume name was given for "%s"' % rname
		res.addAttribute('lv_name', lv_name)
	except Exception, e:
		lv_name = None

	if not lv_name and not vg_name:
		errors.append('Neither lv_name nor vg_name given for resource "%s"' % rname)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addLVM6: no vg_name or lv_name for "%s"' % rname)

	if form.has_key('self_fence'):
		res.addAttribute('self_fence', '1')
	else:
		res.removeAttribute('self_fence')
	return errors

def addSAPDatabase(res, rname, form, model):
	errors = list()

	res.removeAttribute('name')
	res.addAttribute('SID', rname)

	try:
		dbtype = form['DBTYPE'].strip()
		if not dbtype in ( 'ORA', 'DB6', 'ADA' ):
			raise Exception, 'You gave an invalid database type: %s' % dbtype
		res.addAttribute('DBTYPE', dbtype)
	except Exception, e:
		err = str(e)
		errors.append(err)
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('addSAPD1: %s' % err)

	for param in ( 'DBJ2EE_ONLY', 'AUTOMATIC_RECOVER', 'STRICT_MONITORING' ):
		if form.has_key(param):
			res.addAttribute(param, 'true')
		else:
			res.removeAttribute(param)
			
	# Optional string parameters
	for param in ( 'DIR_EXECUTABLE', 'NETSERVICENAME', 'DIR_BOOTSTRAP', 'DIR_SECSTORE', 'JAVA_HOME', 'DB_JARS', 'PRE_START_USEREXIT', 'POST_START_USEREXIT', 'PRE_STOP_USEREXIT', 'POST_STOP_USEREXIT' ):
		try:
			pval = form[param].strip()
			if not pval:
				raise KeyError, 'blank'
			res.addAttribute(param, pval)
		except KeyError, e:
			res.removeAttribute(param)
		except Exception, e:
			err = str(e)
			errors.append(err)
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('addSAPD2: %s %s' % (param, err))
	return errors

def addSAPInstance(res, rname, form, model):
	errors = list()

	res.removeAttribute('name')
	res.addAttribute('InstanceName', rname)

	if form.has_key('AUTOMATIC_RECOVER'):
		res.addAttribute('AUTOMATIC_RECOVER', 'true')
	else:
		res.removeAttribute('AUTOMATIC_RECOVER')

	# Optional string parameters
	for param in ( 'DIR_EXECUTABLE', 'DIR_PROFILE', 'START_PROFILE', 'PRE_START_USEREXIT', 'POST_START_USEREXIT', 'PRE_STOP_USEREXIT', 'POST_STOP_USEREXIT', 'START_WAITTIME' ):
		try:
			pval = form[param].strip()
			if not pval:
				raise KeyError, 'blank'
			res.addAttribute(param, pval)
		except KeyError, e:
			res.removeAttribute(param)
		except Exception, e:
			err = str(e)
			errors.append(err)
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('addSAPI1: %s %s' % (param, err))
	return errors

def config_resource(params, res, rname, form):
	errors = list()

	fvar = GetFormVars(form, map(lambda x: x[0], params))
	for i in params:
		name = i[0]
		val = fvar[name]
		required = i[2]
		default_val = i[3]

		if val is None:
			if required is True:
				if default_val is None:
					errors.append('No value for %s was given for resource "%s"' % (i[1], rname))
				else:
					res.addAttribute(name, default_val)
			else:
				try:
					res.removeAttribute(name)
				except:
					pass
		else:
			res.addAttribute(name, val)
	return errors

def addSybaseASE(res, rname, form, model):
	# parameter (str), short desc (str), required (bool), default (str or None)
	params = (
		('sybase_home', 'SYBASE home directory', True, None),
		('server_name', 'ASE server name', True, None),
		('login_file', 'Login file', True, None),
		('interfaces_file', 'Interfaces file', True, None),
		('sybase_ase', 'SYBASE_ASE directory name', True, 'ASE-15_0'),
		('sybase_ocs', 'SYBASE_OCS directory name', True, 'OCS-15_0'),
		('sybase_user', 'Sybase user', True, 'sybase'),
		('shutdown_timeout', 'Shutdown timeout value', True, '0'),
		('start_timeout', 'Start timeout value', True, '0'),
		('deep_probe_timeout', 'Deep probe timeout value', True, '0')
	)
	errors = config_resource(params, res, rname, form)
	return errors

def addOracleDB(res, rname, form, model):
	params = (
		('user', 'Oracle User Name', True, None),
		('home', 'Oracle Home Directory', True, None),
		('oracletype', 'Oracle Installation Type', False, None),
		('vhost', 'Virtual Hostname', False, None)
	)
	errors = config_resource(params, res, rname, form)
	dbtype = res.getAttribute('oracletype')
	res.removeAttribute('type')
	if dbtype:
		res.removeAttribute('oracletype')
		dbtype = dbtype.lower()
		if dbtype in ('base', 'base-em', '10g', 'ias', '10g-ias'):
			res.addAttribute('type', dbtype)
	return errors

def addOracleInstance(res, rname, form, model):
	params = (
		('user', 'Oracle User Name', True, None),
		('home', 'Oracle Home Directory', True, None),
		('listeners', 'Oracle listeners', False, None),
		('lockfile', 'Path to lock file', False, None),
	)
	errors = config_resource(params, res, rname, form)
	return errors

def addOracleListener(res, rname, form, model):
	params = (
		('user', 'Oracle User Name', True, None),
		('home', 'Oracle Home Directory', True, None),
	)
	errors = config_resource(params, res, rname, form)
	return errors

resource_table = {
	'ip':			( addIp,				Ip				),
	'fs':			( addFs,				Fs				),
	'gfs':			( addClusterfs,			Clusterfs		),
	'nfsm':			( addNetfs,				Netfs			),
	'nfsx':			( addNFSExport,			NFSExport		),
	'nfsc':			( addNFSClient,			NFSClient		),
	'scr':			( addScript,			Script			),
	'smb':			( addSamba,				Samba			),
	'tomcat-5':		( addTomcat5,			Tomcat5			),
	'postgres-8':	( addPostgres8,			Postgres8		),
	'apache':		( addApache,			Apache			),
	'openldap':		( addOpenLDAP,			OpenLDAP		),
	'lvm':			( addLVM,				LVM				),
	'mysql':		( addMySQL,				MySQL			),
	'SAPDatabase':	( addSAPDatabase,		SAPDatabase		),
	'SAPInstance':	( addSAPInstance,		SAPInstance		),
	'oracledb':		( addOracleDB,			OracleDB		),
	'orainstance':	( addOracleInstance,	OracleInstance	),
	'oralistener':	( addOracleListener,	OracleListener	),
	'ASEHAagent':	( addSybaseASE,			SybaseASE		)
}

def create_resource(res_type, form, model):
	errors = list()
	if not resource_table.has_key(res_type):
		raise Exception, [ 'Unknown resource type: "%s"' % res_type ]

	res = None
	if form.has_key('edit'):
		if not form.has_key('oldname'):
			raise Exception, [ 'Cannot find this resource\'s original name.' ]

		oldname = form['oldname'].strip()
		if not oldname:
			raise Exception, [ 'Cannot find this resource\'s original name.' ]

		try:
			res = getResourceForEdit(model, oldname)
			if not res:
				raise Exception, 'not found'
		except Exception, e:
			raise Exception, [ 'No Resource named "%s" exists.' % oldname ]
	else:
		res = resource_table[res_type][1]()

	if res_type != 'ip':
		if not form.has_key('resourcename') or not form['resourcename'].strip():
			raise Exception, [ 'All resources must have a unique name.' ]
		rname = form['resourcename'].strip()
		res.addAttribute('name', rname)
	else:
		rname = form['ip_address'].strip()

	if not form.has_key('edit'):
		try:
			dummy = getResourceForEdit(model, rname)
			if dummy:
				errors.append('A resource named "%s" already exists.' % rname)
		except:
			pass

	if len(errors) > 0:
		raise Exception, errors

	errors = resource_table[res_type][0](res, rname, form, model)

	if len(errors) > 0:
		raise Exception, errors

	return res
