# Copyright (C) 2006-2008 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from ricci_communicator import RicciCommunicator
from RicciQueries import list_services, updateServices, svc_manage
from LuciSyslog import get_logger
from conga_constants import LUCI_DEBUG_MODE
from xml.dom import minidom

luci_log = get_logger()

def get_sys_svc_list(self, request, hostname):
	try:
		rc = RicciCommunicator(hostname)
		if not rc:
			raise Exception, 'None'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('GSSL0: %s: %r %s' % (hostname, e, str(e)))
		return []

	service_list = list_services(rc)
	if not service_list:
		return []

	svc_ret = list()
	for i in service_list:
		s = {}
		try:
			name = str(i.getAttribute('name'))
		except:
			name = None
		s['name'] = name

		try:
			desc = i.getAttribute('description')
		except:
			desc = None
		s['desc'] = desc

		try:
			enabled = i.getAttribute('enabled').lower()
			if enabled == 'true':
				s['enabled'] = True
			else:
				raise Exception, 'not enabled'
		except:
			s['enabled'] = False

		try:
			running = i.getAttribute('running').lower()
			if running == 'true':
				s['running'] = True
				s['state'] = 'Running'
			else:
				raise Exception, 'not running'
		except Exception, e:
			s['running'] = False
			s['state'] = 'Stopped'

		s['starturl'] = '%s?pagetype=91&svcname=%s&systemname=%s&operation=start' % (request['URL'], s['name'], hostname)
		s['stopurl'] = '%s?pagetype=91&svcname=%s&systemname=%s&operation=stop' % (request['URL'], s['name'], hostname)
		s['restarturl'] = '%s?pagetype=91&svcname=%s&systemname=%s&operation=restart' % (request['URL'], s['name'], hostname)
		svc_ret.append(s)

	return svc_ret

def validate_svc_update(self, request):
	try:
		form = request.form
		hostname = form['systemname'].strip()
		if not hostname:
			raise Exception, 'blank'
	except Exception, e:
		return (False, {'errors': [ 'No system name was given.' ]})

	try:
		rc = RicciCommunicator(hostname)
		if not rc:
			raise Exception, 'unknown error'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('VSU0: %s: %r %s' % (hostname, e, str(e)))
		return (False, {'errors': [ 'Unable to connect to the ricci agent on %s: %s' % (hostname, str(e)) ]})

	sys_svc_list = list()
	sys_svc_hash = dict()

	try:
		for i in list_services(rc):
			svc_name = str(i.getAttribute('name'))
			sys_svc_hash[svc_name] = i
			sys_svc_list.append(svc_name)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('VSU1: %s: %r %s' % (hostname, e, str(e)))
		return (False, {'errors': [ 'Unable to retrieve the list of services from %s' % hostname ]})

	try:
		del form['pagetype']
		del form['systemname']
	except:
		pass

	disable_list = filter(lambda x: not form.has_key(x) and sys_svc_hash[x].getAttribute('enabled').lower() == 'true', sys_svc_list)
	enable_list = filter(lambda x: form.has_key(x) and sys_svc_hash[x].getAttribute('enabled').lower() != 'true', sys_svc_list)

	updateServices(rc, enable_list, disable_list)
	request.RESPONSE.redirect(request['URL'] + '?pagetype=90&systemname=' + hostname)

def validate_manage_svc(self, request):
	ret = minidom.Document()

	result = ret.createElement('result')
	result.setAttribute('success', '0')

	servicename = None
	try:
		servicename = request['svcname'].strip()
		result.setAttribute('service', servicename)
	except:
		result.setAttribute('service', 'this service')

	op = None
	try:
		op = request['operation'].strip()
	except:
		result.setAttribute('operation', 'manage')

	try:
		hostname = request['systemname'].strip()
		if not hostname:
			raise Exception, 'blank'
	except Exception, e:
		hostname = None
		result.setAttribute('message', 'No system was specified')
		ret.appendChild(result)

	if hostname and op and servicename:
		try:
			rc = RicciCommunicator(hostname)
			if not rc:
				raise Exception, 'none'
			ret = svc_manage(rc, hostname, servicename, op)
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('VMS0: %r %s' % (e, str(e)))
			result.setAttribute('message', str(e))
			ret.appendChild(result)
	else:
		ret.appendChild(result)

	request.RESPONSE.setHeader('Content-Type', 'text/xml; charset=UTF-8')
	request.RESPONSE.setHeader('Cache-Control', 'no-cache, no-store, private')
	request.RESPONSE.write(str(ret.toxml()))
